<?php
/*
 * Created on 27 Feb. 2007
 *
 * @author Arnaud Spicht
 * @copyright 2007 Arkeia Software
 */

require_once "Arkeia/Soap.php";


class ArkObject {

    protected $_model = null; 
    protected $_mvc = null; 

    public function __construct($mvc=null, $model=null)
    {
        if ($model) $this->_model = $model;
        else $this->_model = get_class($this);
        $this->_mvc = $mvc;
    }

    protected function reconnect($sessnum=1)
    {
        require_once 'Zend/Auth.php';
        $auth = Zend_Auth::getInstance();
        $id = $auth->getIdentity();
        if (!$id) return null;
        $username = $id['username'];
        $password = $id['password'];
        $admin = $id['admin'];
        return $this->connect($auth, $username, $password, $admin, $sessnum);
    }

    public function connect($auth, $username, $password, $admin, $sessnum = 0)
    {
        $keepit = true;
        $acx = Arkeia_Soap::getInstance();
        if ($sessnum == 0) {
            $sessnum = $acx->getAndReserveSessnum();
            $keepit = false;
        }
        require_once 'Arkeia/Auth/Adapter/Arkws.php';
        $authenticator = new Arkeia_Auth_Adapter_Arkws($username, $password, $admin, $sessnum);
        $result = $auth->authenticate($authenticator);
        if (!$keepit) $acx->freeSessnum($sessnum);
        return $result;
    }

    public function new_trans($trans, $trans_version, $asked_sessnum, $vector, $context=null)
    {
        $acx = Arkeia_Soap::getInstance();
        $first = 1;
        if ($context) $soap_context = $acx->list2soap($context);
        else $soap_context = $context;

        $sessnum = $acx->getAndReserveSessnum();
        $session_arkeia = Zend_Registry::get('session_arkeia');
        $sessid = 'session_id_' . (string)$sessnum;
        while ($first) {
            if (!$session_arkeia->$sessid) {
                $first = $first + 1;
                $res = $this->reconnect($sessnum);
                if (!$res) continue;
                if ($res->isValid()) continue;
		$result = $acx->_last_result;
            } else {
                $result = $acx->new_trans(
                                $trans, 
                                $trans_version, 
                                $sessnum, 
                                $vector,
                                $soap_context);
                require_once 'Zend/Json.php';
            }
            if (is_soap_fault($result)) {
                if ($result->faultcode == "Server" && 
                        $result->faultstring == 1 &&
                        $first == 1) {
                    $first = $first + 1;
                    $res = $this->reconnect($sessnum);
                    if (!$res) continue;
                    if ($res->isValid()) continue;
		    $result = $acx->_last_result;
		    if (!is_soap_fault($result)) {
		      return $acx->checkLast($result, $sessnum);
		    }
                }
                $f = Zend_Registry::get('flashMessenger');
                $f->addMessage($result->detail);
                if (!$sessnum) $acx->freeSessnum($sessnum);
                return null;
            }
            return $acx->checkLast($result, $sessnum);
        }
        if (!$sessnum) $acx->freeSessnum($sessnum);
    }

    protected function list2soap($orig_list)
    {
        $acx = Arkeia_Soap::getInstance();
        return $acx->list2soap($orig_list);
    }

    public function getNext($transnum, $sessnum="1", $context=null)
    {
        $acx = Arkeia_Soap::getInstance();
        $result = $acx->getNext($transnum, $sessnum, $context);
        return $acx->checkLast($result, $sessnum);
    }

    public function abortTrans($transnum, $sessnum="1", $context=null)
    {
        $acx = Arkeia_Soap::getInstance();
        $result = $acx->abortTrans($transnum, $sessnum);
        return $result;
    }

    public function getResultAll($context=null)
    {
        if (!$this->_mvc || !$this->_model) return null;
        $trans = $this->_mvc->model->{$this->_model}->getAll;
        $trans_version = $this->_mvc->model->{$this->_model}->getAllVersion;
        if (!$trans_version) $trans_version= 1;
        $snd_all = 1;

        $vector = array("param0" => $snd_all);
        $result = $this->new_trans($trans, $trans_version, 2, $vector, $context);
        return $result;
    }

    public function getResultAllIntersect($context=null)
    {
        if (!$this->_mvc || !$this->_model) return null;
        $trans = $this->_mvc->model->{$this->_model}->getAll.'_INTERSECT';
        $trans_version = $this->_mvc->model->{$this->_model}->getAllVersion;
        if (!$trans_version) $trans_version= 1;
        $snd_all = 1;

        $vector = array("param0" => $snd_all);
        $result = $this->new_trans($trans, $trans_version, 2, $vector, $context);

        /* return use for combobox and combobox can't call getNext */
        while ($result->param0 == '12' && $result->param1 == '2') {
            sleep(1);
            $result = $this->getNext($result->transnum,
                                     $result->sessnum,
                                     $context);
        }

        //error_log('getResultAllIntersect :'.Zend_Json::encode($result));
        return $result;
    }

    public function getAll($context=null)
    {
        $result = $this->getResultAll($context);
        if ($result == null) return null;
        $data = $result->param2->item;
        if (!$data) return array();

        if (!is_array($data)) return array($data);
        return $data;
    }

    /**
     * Call transaction specified in config/mvc.ini to get data for one
     * Arkeia object (like Tape or Savepack).
     * This is a generic function. It can be redefined in a model class.
     * See ArkPool for an example.
     */
    public function getResultOne($id, $context=null)
    {
        if (!$this->_mvc || !$this->_model) return null;
        $trans = $this->_mvc->model->{$this->_model}->getOne;
        $trans_version = $this->_mvc->model->{$this->_model}->getOneVersion;

        if (!$trans_version) $trans_version = 1;
        $vector = array("param0" => $id);
        return $this->new_trans($trans, $trans_version, 1, $vector,$context);
    }

    public function getOne($id, $context=null)
    {
        $result = $this->getResultOne($id, $context);
        if ($result == null) return null;
        if (is_array($result)) {
            foreach ($result as $key => $val) {
                if ($key == "local") continue;
                if ($context && $result[$key]->param0 == "0") continue;
                if ($context) {
                    if(isset($result[$key]->param3)) {
                        $data = $result[$key]->param3;
                    }
                } else {
                    if(isset($result[$key]->param2)) {
                        $data = $result[$key]->param2;
                    }
                }
                if(isset($data)) {
                    return $data;
                }
            }
        } else {
            if ($context) {
                $data = $result->param3;
            } else {
                $data = $result->param2;
            }
            return $data;
        }
        return null;
    }

    public function removeObjects($list, $context=null)
    {
        if (!$this->_mvc || !$this->_model) return null;
        $trans = $this->_mvc->model->{$this->_model}->removeObject;
        $trans_version = $this->_mvc->model->{$this->_model}->removeObjectVersion;
        if (!$trans_version) $trans_version= 1;
        $vector = array("param0" => $this->list2soap($list));
        $result = $this->new_trans($trans, $trans_version, 1, $vector, $context);
        return $result;
    }

    public function removeObject($id, $context=null)
    {
        if (!$this->_mvc || !$this->_model) return null;
        $trans = $this->_mvc->model->{$this->_model}->removeObject;
        $trans_version = $this->_mvc->model->{$this->_model}->removeObjectVersion;
        if (!$trans_version) $trans_version= 1;
        $vector = array("param0" => $id);
        $result = $this->new_trans($trans, $trans_version, 1, $vector,$context);
        return $result;
    }

    public function createObject($list, $context=null)
    {
        if (!$this->_mvc || !$this->_model) return null;
        $trans = $this->_mvc->model->{$this->_model}->createObject;
        $trans_version = $this->_mvc->model->{$this->_model}->createObjectVersion;
        if (!$trans_version) $trans_version = 1;
        $vector = array("param0" => $this->list2soap($list));
        $result = $this->new_trans($trans, $trans_version, 1, $vector,$context);
        return ($result);
    }

    public function modifyObject($list, $context=null)
    {
        if (!$this->_mvc || !$this->_model) return null;
        $trans = $this->_mvc->model->{$this->_model}->modifyObject;
        $trans_version = $this->_mvc->model->{$this->_model}->modifyObjectVersion;
        if (!$trans_version) $trans_version = 1;
        $vector = array("param0" => $this->list2soap($list));
        $result = $this->new_trans($trans, $trans_version, 1, $vector,$context);
        return ($result);
    }

    public function add_to_resultOne($result, $addon, $itemname, $context=null)
    {
        require_once 'Zend/Json.php';

        if (!$result) return $result;

        $item = null;
        foreach ($addon as $appid => $data) {
            $item = $data['PARAM2'][0]->item;
            if ($item && !is_array($item)) {
                $item = array($item);
            }
        }

        if (!is_array($result)) {
            if ($context) {
                $result->param3->$itemname = $item;
            } else {
                $result->param2->$itemname = $item;
            }
            return $result;
        }
        foreach ($result as $key => $val) {
            if ($context && ($key == "local" || $val->param0 == '0')) continue;
            $result[$key]->param3->$itemname = $item;
        }
        return $result; 
    }

    protected function _transform_items($result, $context, $func)
    {
        if ($context) $param = param3;
        else $param = param2;
        if (is_array($result)) {
            foreach ($result as $key => $val) {
                if ($context && $key == "local") continue;
                $data = $result[$key]->$param->item;
                if (!$data) continue;
                if (is_array($data)) {
                    foreach ($data as $item) {
                        $func($this, $item);
                    }
                } else { /* only one item */
                    $func($this, $data);
                }
            }
        } else {
            $data = $result->$param->item;
            if (!$data) return $result;
            if (is_array($data)) {
                foreach ($data as $item) {
                    $func($this, $item);
                }
            } else { /* only one item */
                $func($this, $data);
            }
        }
        return $result;
    }

     protected function _transform_globals($result, $context, $func)
    {
        if ($context) $param = param3;
        else $param = param2;
        if (is_array($result)) {
            foreach ($result as $key => $val) {
                if ($context && $key == "local") continue;
                $data = $result[$key]->$param;
                if (!$data) continue;
                $func($this, $data);
            }
        } else {
            $data = $result->$param;
            if (!$data) return $result;
            $func($this, $data);
        }
        return $result;
    }

    /* _filter_one_item will change the result to get only one item
       $func is a compare function to find the item.
       Each item is passed to this function.
       See ArkJob/getResultOne for an example.
    */
    protected function _filter_one_item($result, $context, $func)
    {
        if ($context) $param = param3;
        else $param = param2;
        if (is_array($result)) {
            foreach ($result as $key => $val) {
                if ($context && $key == "local") continue;
                $data = $result[$key]->$param->item;
                if (is_array($data)) {
                    foreach ($data as $item) {
                        if ($func($item)) {
                            $result[$key]->$param = $item;
                            return $result;
                        }
                    }
                } else if ($func($data)) {
                    $result[$key]->$param = $data;
                    return $result;
                }
                $result[$key]->$param = null;
            }
        } else {
            $data = $result->$param->item;
            if (is_array($data)) {
                foreach ($data as $item) {
                    if ($func($item)) {
                        $result->$param = $item;
                        return $result;
                    }
                }
            } else if ($func($data)) {
                $result->$param = $data;
                return $result;
            }
            $result->$param = null;
        }
        return null;
    }

    public function getstatus($context=null) {

        $trans = 'INFO_STATUS';
        $trans_version = '2';
        $vector = array();

         $result = $this->new_trans($trans, $trans_version, 1, $vector,$context);

        return $result;

    }

}
