/**
 * Arkeia Backup Javascript 
 * 
 * Actions, Controls, and Event handlers
 * 
 * TODO: Move here all backup functionalities from :
 *  - arkeia.widget.ArkeiaInfo
 *  - arkeia.layout.verifForm
 *  - arkeia.layout.transformData
 *  - any others
 */


dojo.provide('arkeia.backup');

dojo.require('arkeia.init');;

arkeia.backup = {_name: '[arkeia.backup]'};


/**
 * arkeia.backup.changeValueScreen
 * 
 * Change the unit value to the fields of the backup screen
 * 
 * => Called by the widget "bkp_change_speed" (arkeia:ArkeiaComboBox) 
 *    from "showbackup.phtml"
 * 
 * @param value {String} Unit value
 * @return {Boolean} success
 */
arkeia.backup.changeValueScreen = function (value) {
    var _name = '[arkeia.backup.changeValueScreen] ';
    var list = {
        bkField: {}
    };
    var byId = {
        bkField: {}
    };
    var id = '';

    arkeia.group(_name);
    arkeia.log('(value:', value, ')');

    if (this.widgetId !== 'bkp_change_speed') {
        arkeia.error('Function only defined for "bkp_change_speed" widget, not for : ', this);
        arkeia.groupEnd();
        return false;
    }
    list.bkField = [
        'BK_DSK_AVG_METER', 'BK_DSK_INS_METER', 'BK_PHY_AVG_METER', 'BK_PHY_INS_METER', 
        'BK_DSK_AVG',       'BK_DSK_INS',       'BK_PHY_AVG',       'BK_PHY_INS', 
        'FLOW_DSK_AVG',     'FLOW_DSK_INS',     'FLOW_PHY_AVG',     'FLOW_PHY_INS',
        'DRV_DSK_AVG',      'DRV_DSK_INS',      'DRV_PHY_AVG',      'DRV_PHY_INS'
    ];
    for (id in list.bkField) {
        byId.bkField = digit.byId(list.bkField[id]);
        if (!byId.bkField) {
            arkeia.error('Widget "' + list.bkField[id] + '" not found');
        } else {
            byId.bkField.unitValue = value;
        }
    }
    arkeia.groupEnd();
    return true;
};


/**
 * arkeia.backup.changecruise
 * 
 * Send directly the change cruise command to the server when 
 * a new cruise value is selected from the ArkeiaComboBox
 * (O for unlimited)
 * 
 * @param {Number} value not used because found in the form on submit
 * @return {Boolean} success
 */
arkeia.backup.changecruise = function (value) {
    var _name = '[arkeia.backup.changecruise] ';
    var byId = {
        bkSubmitCruise: {}
    };

    arkeia.group(_name);
    arkeia.log('(value:', value, ')');

    byId.bkSubmitCruise = digit.byId('SUBMITCRUISE');
    if (!byId.bkSubmitCruise) {
        arkeia.error('Widget "SUBMITCRUISE" not found');
        arkeia.groupEnd();
        return false;
    }
    byId.bkSubmitCruise.onClick();
    arkeia.groupEnd();
    return true;
};


/**
 * arkeia.backup.deladvparam
 * 
 * Delete Advanced parameter from the list
 * 
 * @param {String} name part of the id of the dom content element to delete from the advanced params list
 * @return {Boolean} success
 */
arkeia.backup.deladvparam = function (name) {
    var _name = '[arkeia.backup.deladvparam] ';
    var byId = {
        bkContent: {},
        bkAdvparamlist: {}
    };

    arkeia.group(_name);
    arkeia.log('(name:', name, ')');

    byId.bkContent = dojo.byId('content_' + name);
    if (!byId.bkContent) {
        arkeia.error('HTML element "content_' + name + '" not found by id');
        arkeia.groupEnd();
        return false;
    }
    byId.bkAdvparamlist = byId.bkContent.parentNode;
    byId.bkAdvparamlist.removeChild(byId.bkContent);
    arkeia.groupEnd();
    return true;
};


/**
 * arkeia.backup.checkadvparam
 * 
 * Write 'Parameter' and 'Value' as labels
 * in correspondant input text 
 * when they are empty and have not the focus
 * 
 * @param {String} id html element id of the container of param to check
 * @param {String} even event 'focus' or 'blur' determining wich action to do
 * @param {Object} [prefix] facultative prefix for a group of advanced parameters
 * 
 * @return {Boolean} success
 */
arkeia.backup.checkadvparam  = function (id, even, prefix) {
    var _name = '[arkeia.backup.checkadvparam]';
    var byId = {
        bkAdvparam: {}
    };
    var param_lang = '';
    var param_val_lang = '';

    arkeia.group(_name);
    arkeia.log('(id:', id, 'even:', even, 'prefix:', prefix, ')');

    if (!prefix) {
        prefix = '';
    }
    byId.bkAdvparam = dojo.byId(id);
    if (!byId.bkAdvparam) {
        arkeia.error('HTML element "' + id + '" not found by id');
        arkeia.groupEnd();
        return false;
    }
    param_lang = arkeia.lang.getMsg('bkp_inter_form_adv_param_parameter'); 
    param_val_lang = arkeia.lang.getMsg('bkp_inter_form_adv_param_value'); 
    if (even === 'focus') {
        if ((id === (prefix + 'advp_name') && byId.bkAdvparam.value === param_lang) 
         || (id === (prefix + 'advp_val') && byId.bkAdvparam.value === param_val_lang)) {
            byId.bkAdvparam.value = '';
        }
    } else if (even === 'blur') {
        if (id === (prefix + 'advp_name') && byId.bkAdvparam.value === '') {
            byId.bkAdvparam.value = param_lang;
        }
        if (id === (prefix + 'advp_val') && byId.bkAdvparam.value === '') {
            byId.bkAdvparam.value = param_val_lang;
        }
    }
    arkeia.groupEnd();
    return true;
};


/**
 * arkeia.backup.clearadvparam
 * 
 * Delete all Advanced parameters
 * 
 * @return {Boolean} success
 */
arkeia.backup.clearadvparam = function () {
    var _name = '[arkeia.backup.clearadvparam] ';
    var byId;

    arkeia.group(_name);

    byId = dojo.byId('advparamlist');
    if (!byId) {
        arkeia.error('HTML element "advparamlist" not found by id');
        arkeia.groupEnd();
        return false;
    }
    byId.innerHTML = '';
    arkeia.groupEnd();
    return true;
};


/**
 * arkeia.backup.addadvparam
 * 
 * Add a new Advanced parameter
 * 
 * @param {String} name
 * @param {String} value
 * @param {String} [prefix]
 * @return {Boolean} success
 */
arkeia.backup.addadvparam = function (name, value, prefix) {
    var _name = '[arkeia.backup.addadvparam]';
    var byId = {};
    var success = true;
    var node = {};
    var elem = '';
    var content = '';

    arkeia.group(_name);
    arkeia.log('(name:', name, 'value:', value, 'prefix:', prefix, ')');

    if (!prefix) {
        prefix = '';
    }
    if (!name) {
        byId.NAME = dojo.byId(prefix + 'advp_name');
        if (!byId.NAME) {
            arkeia.error('HTML element "' + prefix + 'advp_name" not found by id');
            arkeia.groupEnd();
            return false;
        }
        name = byId.NAME.value;
    }
    if (!value) {
        byId.VALUE = dojo.byId(prefix + 'advp_val');
        if (!byId.VALUE) {
            arkeia.error('HTML element "' + prefix + 'advp_val" not found by id');
            arkeia.groupEnd();
            return false;
        }
        value = byId.VALUE.value;
    }
    /*
    node.paramAction = document.createElement('a');
    node.paramName   = document.createElement('strong');
    node.paramValue  = document.createElement('span');
    node.paramInput  = document.createElement('input');
    node.paramAction.style.cursor = 'pointer';
    dojo.html.setClass(node.paramName, 'name');
    dojo.html.setClass(node.paramValue, 'value');
    // action remove
    node.paramAction.setAttribute('onclick', 'arkeia.layout.backup.deladvparam(name);');
    // text
    node.paramAction.innerHTML = arkeia.lang.getMsg('bkp_inter_form_adv_param_remove');
    node.paramName.innerHTML = name;
    node.paramValue.innerHTML = value;
    // hidden input
    node.paramInput.setAttribute('type', 'hidden');
    node.paramInput.setAttribute('name', prefix + 'adv_param_' + name);
    node.paramInput.setAttribute('value', value);
    */
    content  = '<a href="javascript:;" onclick="arkeia.layout.backup.deladvparam(\'' + name + '\')" style="cursor: pointer;">'
             + arkeia.lang.getMsg('bkp_inter_form_adv_param_remove')
             + '</a> <strong class="name">' + name + '</strong>:'
             + ' <span class="value">"' + value + '"</span>'
             + '<input type="hidden" name="' + prefix + 'adv_param_' + name + '" value="' + value + '" />';
    byId.CONTENT = dojo.byId('content_' + name);
    if (byId.CONTENT) {
        /*
        if (byId.CONTENT.childNodes) {
            for (elem in byId.CONTENT.childNodes) {
                byId.CONTENT.removeChild(byId.CONTENT.childNodes[elem]);
            }
        }
        // display node
        byId.CONTENT.appendChild(node.paramAction);
        byId.CONTENT.appendChild(node.paramName);
        byId.CONTENT.appendChild(node.paramValue);
        byId.CONTENT.appendChild(node.paramInput);
        */
        byId.CONTENT.innerHTML = content;
    } else {
        byId.PARAMS = dojo.byId(prefix + 'advparamlist');
        if (!byId.PARAMS) {
            arkeia.error('HTML elements "content_' + name + '" and "' + prefix + 'advparamlist" not found by id');
            success = false;
        } else {
            /*
            if (byId.PARAMS.childNodes) {
                for (elem in byId.PARAMS.childNodes) {
                    byId.PARAMS.removeChild(byId.PARAMS.childNodes[elem]);
                }
            }
            node = document.createElement('div');
            dojo.debug(_name + 'in html element : "' + prefix + 'advparamlist" <div> has a name Attribute !!!');
            node.setAttribute('name', 'advparam');
            node.setAttribute('id', 'content_' + name);
            node.innerHTML = content;
            byId.PARAMS.appendChild(node);
            */
            arkeia.warn(' In the HTML element : "' + prefix + 'advparamlist" <div> has a name Attribute !!!');
            content =  '<div name="advparam" id="content_' + name + '">' + content + '</div>';
            byId.PARAMS.innerHTML += content;
        }
    }
    if (byId.NAME) {
        byId.NAME.value = arkeia.lang.getMsg('bkp_inter_form_adv_param_parameter');
    }
    if (byId.VALUE) {
        byId.VALUE.value = arkeia.lang.getMsg('bkp_inter_form_adv_param_value');
    }
    arkeia.groupEnd();
    return success;
};


/**
 * arkeia.backup.TypeChangeValue
 * 
 * Used when value changed by user
 * 
 * @param {String} name
 * @param {String} value
 * @param {String} [prefix]
 * @return {Boolean} success
 */
arkeia.backup.TypeChangeValue = function (_this, value) {
    var _name = '[arkeia.backup.TypeChangeValue]';
    var byId = {};
    var success = true;

    arkeia.group(_name);
    arkeia.log('(_this:', _this, 'value:', value, ')');

    if (!_this || !_this.listened) {
        arkeia.error(' "_this" or "_this.listened" is undefined ');
        success = false;
    } else {
        byId = digit.byId(_this.listened);
        if (!byId) {
            arkeia.error(' widget "' + _this.listened + '" not found');
            success = false;
        } else {
            _this.unlistenArkeiaParent(byId);
        }
    }
    if (value === 'CONTINUOUS' || value === 'STANDARD') {
        byId = digit.byId('SUBMIT_' + value);
        if (!byId) {
            arkeia.error(' widget "SUBMIT_' + value + '" not found');
            success = false;
        } else {
            byId.onClick();
        }
    }
    arkeia.groupEnd();
    return success;
};


// back compatibility aliases
arkeia.layout.backup = arkeia.backup;

changeValueScreen = arkeia.backup.changeValueScreen;
bkp_changecruise = arkeia.backup.changecruise;
bkp_deladvparam = arkeia.backup.deladvparam; 
bkp_checkadvparam = arkeia.backup.checkadvparam;
bkp_clearadvparam = arkeia.backup.clearadvparam;
bkp_addadvparam = arkeia.backup.addadvparam;
bkpTypeChangeValue = arkeia.backup.TypeChangeValue;

arkeia.info('[arkeia.backup] loaded : ', arkeia.backup);