<?php
/*
 * Created on 05 Mar. 2007
 *
 * @author Arnaud Spicht
 * @copyright 2007 Arkeia Software
 */

class Arkeia_Plugin_Auth extends Zend_Controller_Plugin_Abstract 
{
	private $_auth;
	private $_acl;

	private $_noauth = array('controller' => 'index',
				 'action'     => 'summary');

	private $_noacl = array('controller' => 'error',
				'action'     => 'privileges');
	
	public function __construct($auth, $acl)
	{
		$this->_auth = $auth;
		$this->_acl = $acl;
	}

	public function preDispatch($request)
	{
		// Stage 1: check if controller and action exist
		$front = Zend_Controller_Front::getInstance();
		$response = $front->getResponse();

		// Set no cache header

		$response->setHeader("Cache-Control","no-cache");
		$response->setHeader("Pragma", "no-cache");
		$response->setHeader("charset", "UTF-8");


		$dispatcher = $front->getDispatcher();

		$controller = $dispatcher->getControllerClass($request);
		try {
			$dispatcher->loadClass($controller);
		}
		catch (Zend_Exception $e) {
			$controller = $dispatcher->getDefaultControllerClass($request);
		}

		$action = $dispatcher->getActionMethod($request);

		$controllerName = $request->getControllerName();

		if (!method_exists($controller, $action)) {
			$defaultAction = $dispatcher->getDefaultAction();
			$response->setRedirect($front->getBaseUrl() . '/' . $controllerName . '/' . $defaultAction);
			$response->sendHeaders();
			exit;
		}

		if ($this->_auth->hasIdentity()) {
			$id = $this->_auth->getIdentity();
			$role = $id['role'];
		}
		if (!isset($role) || !$role) {
			$role = 'guest';
		}
		$resource = strtolower($controllerName);
		$actionName = $request->getActionName();

		if (!$this->_acl->has($resource)) {
			$resource = null;
		}

		if (!$this->_acl->isAllowed($role, $resource, $actionName)) {
			if (!$this->_auth->hasIdentity()) {
				$controllerName = $this->_noauth['controller'];
				$actionName = $this->_noauth['action'];
				$request->setControllerName($controllerName);
				$request->setActionName($actionName);
			} else {
				$controllerName = $this->_noauth['controller'];
				$actionName = $this->_noauth['action'];
				$request->setControllerName($controllerName);
				$request->setActionName($actionName);
			}
		}
	}
}

?>
