<?php
/*
 * Created on 27 Feb. 2007
 *
 * @author Arnaud Spicht
 * @copyright 2008, Arkeia Software
 * @version   CVS: $Id: Soap.php,v 1.43 2009/03/20 20:16:50 fdaniel Exp $
 */

define('SESS_ID', 'session_id_');
define('SESS_USED', 'session_used_');


class Arkeia_Soap {
    static private $_transnum = 0;

    public $_last_result = null;

    // Singleton management
    static private $_instance = null;

    private function __construct() {
    }

    static public function getInstance() {
        if (!self::$_instance instanceof self) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    // Connection management
    private $soapClient = null;

    private function convert_soap_call($result) {
        require_once 'Zend/Json.php';
        foreach($result as $key => $val) {
            if (is_array($val)) {
                if (is_string($key) && preg_match("/^base64_(.+)$/",$key,$matches)) {
                    $result->$matches[1] = array();
                    foreach($val as $key_2 => $val_2) {
                        $new_val = base64_decode($val_2);
                        $result->{$matches[1]}[$key_2] = $new_val;
                    }
                    unset($result->$key);
                } else {
                    foreach($val as $key_2 => $val_2) {
                        if (is_object($val_2)) {
                            $this->convert_soap_call($val_2);
                        } else if (is_string($val_2) && (preg_match("/^base64_(.+)$/",$key_2,$matches))) {
                            $val[$matches[1]] = base64_decode($val_2);
                            unset($val[$key_2]);
                        }
                    }
                }
            } else if (is_object($val)) {
                $this->convert_soap_call($val);
            } else if (is_string($val) && (preg_match("/^base64_(.+)$/",$key,$matches))) {
                $new_val = base64_decode($val);
                unset($result->$key);
                $result->$matches[1] = $new_val;
            }
        }
    }

    public function getSoapClient() {
        if ($this->soapClient == null) {
            $config = Zend_Registry::get('config');
            $this->soapClient = new SoapClient(null, array(
                'location'   => $config->arkws->location,
                'uri'        => $config->arkws->uri,
                'style'      => SOAP_RPC,
                'use'        => SOAP_ENCODED,
                'trace'      => 1,
                'exceptions' => 0)
            );
        }
        return $this->soapClient;
    }

    public function _rexec($method, $params ) {
        $config = Zend_Registry::get('config');
        $soapClient = $this->getSoapClient();
        $options = array(
            "uri" => $config->arkws->uri,
            "soapaction" => $config->arkws->uri."/".$method
        );
        $result = $soapClient->__soapCall($method,$params,$options);
        $response =  $soapClient->__getLastResponse();
        if ($result) {
            $this->convert_soap_call($result);
        }
        return $result;
    }

    public function login($username, $passwd, $admin=0, $sessnum=1) {
        $config = Zend_Registry::get('config');
        $session_arkeia = Zend_Registry::get('session_arkeia');
        $sessid = SESS_ID . (string)$sessnum;
        if (!$session_arkeia->$sessid) {
            $session_arkeia->$sessid = uniqid();
            $session_arkeia->max_sessnum = max($session_arkeia->max_sessnum, $sessnum);
        }
        $method = "arkws_connect";
        if (!$admin) { $server = $config->arkws->local_server; }
        else { $server = $config->arkws->admin_server; }
        if (!$server) $server = "ARKTRANS";
        $LanguagesEnabled = split(',', $config->arkws->enabled_lang);
        if (isset($_REQUEST['lang']) && in_array($_REQUEST['lang'], $LanguagesEnabled)) {
	    $lang = strtoupper($_REQUEST['lang']);
        } else {
	    $lang = strtoupper($config->arkws->default_lang);
        }
        $params = array(
            new SoapParam( $admin, "admin"),
            new SoapParam( $username, "login"),
            new SoapParam( $passwd, "passwd"),
            new SoapParam( $sessnum, "sess_num"),
            new SoapParam( $config->arkws->remote_host, "remote_host"),
            new SoapParam( $server, "server"),
            new SoapParam( $session_arkeia->$sessid, "sessid"),
            new SoapParam( $lang, "lang")
        );
        return $this->_rexec($method,$params);
    }

    public function logout($sessnum = 1) {
        $session_arkeia = Zend_Registry::get('session_arkeia');
        $sessid = SESS_ID . (string) $sessnum;
        $method = "arkws_disconnect";
        $params = array(
            new SoapParam( $session_arkeia->$sessid, "sessid"),
        );
        return $this->_rexec($method,$params);
    }

    public function logoutAll() {
        $session_arkeia = Zend_Registry::get('session_arkeia');
        for ($sessnum=1 ; $sessnum <= $session_arkeia->max_sessnum; $sessnum++) {
            $sessid = SESS_ID . (string) $sessnum;
            if ($session_arkeia->$sessid) {
                $this->logout($sessnum);
                $session_arkeia->$sessid = null;
            }
        }
    }

    public function logoutAllShutdown($action=null, $time=0) {
        $session_arkeia = Zend_Registry::get('session_arkeia');
        for ($sessnum=1 ; $sessnum<=$session_arkeia->max_sessnum; $sessnum++) {
            $sessid = SESS_ID . (string)$sessnum;
            if ($session_arkeia->$sessid) {
                $this->logoutShutdown($sessnum, $action, $time);
                $session_arkeia->$sessid = null;
            }
        }
    }

    public function logoutShutdown($sessnum=1, $action=null, $time=0) {
        $session_arkeia = Zend_Registry::get('session_arkeia');
        $sessid = SESS_ID . (string)$sessnum;
        $method = "arkws_disconnect";
        $params = array(
            new SoapParam( $session_arkeia->$sessid, "sessid"),
            new SoapParam( "root", "login"),
            new SoapParam( $session_arkeia->passroot, "passwd"),
            new SoapParam( $time, $action)
        );
        return $this->_rexec($method,$params);
    }

    private function b64encode_param_in($value, $inStruct=false) {

        if(in_array(gettype($value), array('string', 'array', 'object'))) {

            switch(gettype($value)) {

                case 'string':
                    $result = base64_encode($value);
                break;

                case 'array':

                    $result = $value;

                    foreach($result as $key=>$val) {

                        $result[$key] = $this->b64encode_param_in($val);

                    }

                break;

                case 'object':

                    $result = clone $value;

                    if(isset($result->enc_value) && is_array($result->enc_value)) {

                        foreach($result->enc_value as $key=>$val) {

                            $result->enc_value[$key] = $this->b64encode_param_in($val, true);

                        }

                    }

                break;

            }

        } else {

            if($inStruct) {

                $result = base64_encode($value);

            } else {

                $result = $value;

            }

        }

        return $result;

    }

    public function new_trans($trans, $trans_version, $sessnum, $vector, $context=null) {
        require_once 'Zend/Json.php';
	//DEBUG error_log(__FILE__.':'.__LINE__.' '.$trans.' version '.$trans_version.' vector '.Zend_Json::encode($vector));
        $session_arkeia = Zend_Registry::get('session_arkeia');
        $method = "arkws_method";
        $params = array();
        if ($context) {
            array_push($params, new SoapParam($this->b64encode_param_in($context), "list_id"));
        }
        array_push($params, new SoapParam( $trans, "transaction"));
        array_push($params, new SoapParam( $trans_version, "trans_version"));
        array_push($params, new SoapParam( ++self::$_transnum, "trans_num"));
        $sessid = 'session_id_' . (string)$sessnum;
        array_push($params, new SoapParam( $session_arkeia->$sessid, "sessid"));

        if(is_object($vector)) {
            $newvector = clone $vector;
        } else {
            $newvector = $vector;
        }

        $newvector = $this->b64encode_param_in($newvector);
        foreach ($newvector as $key => $val) {
            array_push($params, new SoapParam($val, $key));
        }
        $result = $this->_rexec($method, $params);
        if ($result) {
            if (is_array($result)) {
                $result["local"]->transnum = self::$_transnum;
                $result["local"]->sessnum = $sessnum;
            } else {
                $result->transnum = self::$_transnum;
                $result->sessnum = $sessnum;
            }
        }
        require_once 'Zend/Json.php';
        //DEBUG error_log(__FILE__.':'.__LINE__.' Params for ' . $trans . ': ' . Zend_Json::encode($params));
        //DEBUG error_log(__FILE__.':'.__LINE__.' Result for ' . $trans . ': ' . Zend_Json::encode($result));
        if($result->param0 == 13) {
            $result->param0 = 1;
            $result->force_alert = $result->param1;
        }
        if ($result->last == 1 && $result->param0 == 12 && $result->param1 == 2) {
            //DEBUG error_log(__FILE__.':'.__LINE__.' GetNext for ' . $trans . 'params: '. Zend_Json::encode($params) . ' result:' . Zend_Json::encode($result));
            $result = $this->getNext(self::$_transnum, $sessnum);
            //DEBUG error_log(__FILE__.':'.__LINE__.' AFTER getNext '.Zend_Json::encode($result));
            $this->checkLast($result, $sessnum);
        }
        /*
        if (!($result->last == 1 && $result->param0 == 12 && $result->param1 == 2) && $result->param0 == 12) {
            error_log(__FILE__.':'.__LINE__.' transaction '.Zend_Json::encode($result));
        }
        */
	$this->_last_result = $result;
	if ($trans == 'USR_VALIDATE' && $result->param0 == '4') {
	  $this->_last_result->force_reconnect = 1;
	}
        return $result;
    }

    public function getNext($transnum, $sessnum=1, $context = null) {
        require_once 'Zend/Json.php';
        $method = "arkws_get_next";
        $session_arkeia = Zend_Registry::get('session_arkeia');
        $sessid = 'session_id_' . (string)$sessnum;
        $params = array(
            new SoapParam( $transnum, "trans_num"),
            new SoapParam( $session_arkeia->$sessid, "sessid")
        );
        $result = $this->_rexec($method,$params);
        //DEBUG error_log(__FILE__.':'.__LINE__.' Params for ' . getNext . ': ' . Zend_Json::encode($params));
        //DEBUG error_log(__FILE__.':'.__LINE__.' Result for ' . getNext . ': ' . Zend_Json::encode($result));
        if ($result) {
            if (is_array($result)) {
                $result["local"]->sessnum = $sessnum;
                $result["local"]->transnum = $transnum;
            } else {
                $result->transnum = $transnum;
                $result->sessnum = $sessnum;
            }
        }
        return $result;
    }

    public function getLastRequest() {
        $soapClient = $this->getSoapClient();
        return $soapClient->__getLastRequest();
    }

    public function getLastResponse() {
        $soapClient = $this->getSoapClient();
        return $soapClient->__getLastResponse();
    }

    public function abortTrans($transnum, $sessnum=1) {
        $method = "arkws_abort";
        $session_arkeia = Zend_Registry::get('session_arkeia');
        $sessid = 'session_id_' . (string)$sessnum;
        $params = array(
            new SoapParam( $transnum, "trans_num"),
            new SoapParam( $session_arkeia->$sessid, "sessid")
        );
        $result = $this->_rexec($method,$params);
        if ($result) {
            if (is_array($result)) {
                $result["local"]->sessnum = $sessnum;
                $result["local"]->transnum = $transnum;
            } else {
                $result->transnum = $transnum;
                $result->sessnum = $sessnum;
            }
        }
        $this->freeSessnum($sessnum);
        return ($result);
    }

    public function list2soap($orig_list) {
        $mylist = array();
        if ($orig_list) {
            foreach ($orig_list as $key => $val) {
                if (is_array($val)) {
                    array_push($mylist, new SoapVar($val, 
                        SOAP_ENC_OBJECT, "xsd:list", null, "item")
                    );
                } else {
                    $mylist[$key] =  $val;
                }
            }
        }
        return new SoapVar($mylist, SOAP_ENC_OBJECT, "xsd:list");
    }

    public function getAndReserveSessnum() {
        $session_arkeia = Zend_Registry::get('session_arkeia');
        $sessnum = 1;
        while (1) {
            $sessid = SESS_ID . (string) $sessnum;
            $used = SESS_USED . (string) $sessnum;
            if (!isset($session_arkeia->$used) || $session_arkeia->$used == 0) {
                $session_arkeia->$used = 1;
                return $sessnum;
            }
            $sessnum++;
        }
    }

    public function freeSessnum($sessnum) {
        $session_arkeia = Zend_Registry::get('session_arkeia');
        $used = SESS_USED . (string) $sessnum;
        $session_arkeia->$used = 0;
    }

    public function checkLast($result, $sessnum) {
        if (is_array($result)) {
            $last_last = "0";
            foreach ($result as $key => $val) {
                if ($key == 'local') {
                    continue;
                }
                $last_last = $result[$key]->last;
            }
            if ($last_last == "1") {
                $this->freeSessnum($sessnum);
            }
        } else if ($result->last == 1 && ($result->param0 != 12 || $result->param1 != 2)) {
            $this->freeSessnum($sessnum);
        }
        return $result;
    }

} // Class Arkeia_Soap
